#!/usr/bin/env bash 
##################################################
# AppOptics .NET Agent Diagnostics Script
# FILE:         agent-diagnostic-logs.sh
# VERSION:      1.0.0
# DESCRIPTION:  .NET AppOptics Agent script for collection of information and log files
# BUGS:         technicalsupport@solarwinds.com
# COPYRIGHT:    2021 SolarWinds Worldwide, LLC 
# LICENSE:      Apache 2.0; http://www.apache.org/licenses/LICENSE-2.0
# ORGANIZATION: http://appoptics.com
#################################################

PROGNAME=$(basename $0)
WD="$(pwd)"
SCRIPT_ROOT_DIR="$( cd -P "$( dirname "$source" )" && pwd )"
echo "Script directory: ${SCRIPT_ROOT_DIR}"

export LOG_DIR=${WD}
mkdir -p ${LOG_DIR}
echo "Log directory: ${LOG_DIR}"

now()
{
  echo $(date +'%Y-%m-%d-%H_%M')
}

# Redirect all output to the log file

LOG_FILE="${LOG_DIR}/${PROGNAME}.`now`.log"
rm -rf ${LOG_FILE}
exec >  >(tee -ia ${LOG_FILE})
exec 2> >(tee -ia ${LOG_FILE} >&2)

APPOPTICS_SERVICE_KEY_ENV=${APPOPTICS_SERVICE_KEY:-""}
APPOPTICS_HOME_APM_DOTNET_ENV=${APPOPTICS_HOME_APM_DOTNET:-""}

APPOPTICS_HOME_DIR=${APPOPTICS_HOME_APM_DOTNET_ENV}
APPOPTICS_PROFILER=${APPOPTICS_HOME_DIR}/AppOpticsProfiler.so
APPOPTICS_CONFIG_FILE_NAME=agent.config
APPOPTICS_CONFIG_FILE=${APPOPTICS_HOME_DIR}/${APPOPTICS_CONFIG_FILE_NAME}
APPOPTICS_PROFILER_CLASS_ID="{824293AD-22E2-4DAA-BC28-166C140543BE}"
APPOPTICS_LOG_DIR="/var/tmp/appoptics"

AGENT_DISPLAY_NAME="AppOptics .NET Agent"

# Archive settings

OUTPUT_DIR=${WD}
ARCHIVE_NAME="appoptics_agent_logs.`now`.tgz"

# Return error codes

readonly SUCCESS=0
readonly FAILURE=1

# Truth values

__IS_TRUE=1
__IS_FALSE=0

# Sizes

KILO=1024
let MEGA=$KILO*$KILO
let GIGA=$MEGA*$KILO
let TERA=$GIGA*$KILO

# Generate warning when trying to add a big file to archive

BIG_FILE_SIZE=100M

__error() 
{
    printf "* ERROR: %s\n" "$*" 1>&2
}

__info()
{
    printf "*  INFO: %s\n" "$*"
}

__warn() 
{
    printf "*  WARN: %s\n" "$*"
}


exists()
{
    command -v "$1" >/dev/null 2>&1
}

__file_exists()
{
    local check_file=${1}
    if [ -f "${check_file}" ]; then
        echo $__IS_TRUE
    else
        echo $__IS_FALSE
    fi
}

__dir_exists()
{
    local check_directory=${1}
    if [ -d "${check_directory}" ]; then
        echo $__IS_TRUE
    else
        echo $__IS_FALSE
    fi
}
APPOPTICS_IS_INSTALLED=$(__dir_exists "${APPOPTICS_HOME_DIR}")

__file_size()
{
    local file=${1}
    if [ -f "${file}" ]; then
        size=$((du --apparent-size --block-size=1 "$file" 2>/dev/null \
                || gdu --apparent-size --block-size=1 "$file" 2>/dev/null \
                || find "$file" -printf "%s" 2>/dev/null \
                || gfind "$file" -printf "%s" 2>/dev/null \
                || stat --printf="%s" "$file" 2>/dev/null \
                || stat -f%z "$file" 2>/dev/null \
                || wc -c <"$file" 2>/dev/null \
               ) | awk '{print $1}')
        result=$?
        if [ $result -eq 0 ]
        then
            echo $size
        fi
    fi
    echo 0
}

__format_filesize()
{
    local size=${1}
    local formatted=""

    if [ "$size" -gt $TERA ]; then
        let size=$size/$TERA
        formatted="$size TB"
    elif [ "$size" -gt $GIGA ]; then
        let size=$size/$GIGA
        formatted="$size GB"
    elif [ "$size" -gt $MEGA ]; then
        let size=$size/$MEGA
        formatted="$size MB"
    elif [ "$size" -gt $KILO ]; then
        let size=$size/$KILO
        formatted="$size KB"
    elif [ "$size" -gt 0 ]; then
        formatted="$size B"
    fi

    echo "$formatted"
}

__failed()
{
    echo "###################################################"
    echo "An error occured during the collection of logs of ${AGENT_DISPLAY_NAME}."
    echo "Please contact AppOptics customer service for support at technicalsupport@appoptics.com."
    echo "###################################################"
    exit ${FAILURE}
}

__usage() 
{
cat <<EOU

  Usage:  ${0} [script options]

  Script Options:
    -h, --help                Show this help screen
    -o, --output <directory>  (Optional) Specifies output directory for the archive file. By default the archive file is created under current working directory.
EOU
}

__run_on_azure()
{
   local check_website_name=${WEBSITE_SITE_NAME:-""}
   if [ "${check_website_name}" == "" ]; then
       echo $__IS_FALSE
   else
       echo $__IS_TRUE
   fi
}
IS_AZURE=$(__run_on_azure)

# Parse command line arguments

while [ $# -gt 0 ]; do
    case ${1} in
    -h | --help)
        __usage
        exit
        ;;
    -o | --output)
        OUTPUT_DIR=${2}
        shift 2
        continue
        ;;

    *)
    esac
    shift
done


__mask_service_key()
{
    local service_key=${1}
    local search=":"
    local show_len=4
    local pos_semicol=$(expr index ${service_key} ${search})
    local service_key_1=${service_key:0:${show_len}}
    local service_key_2=${service_key:${pos_semicol}-${show_len}-1:${show_len}}
    local service_name=${service_key:${pos_semicol}}
    local service_key_only=${service_key:0:${pos_semicol}}
    local pad_len=$((${#service_key_only}-2*${show_len}-1))
    local pad=$(printf '%*s' "${pad_len}")
    pad=${pad// /*}
    local hidden_service_key=${service_key_1}${pad}${service_key_2}":"${service_name}
    echo ${hidden_service_key}
}

__read_service_key()
{
    if [ "${APPOPTICS_IS_INSTALLED}" == "$__IS_TRUE" ]; then
        local check_file_config=$(__file_exists "${APPOPTICS_CONFIG_FILE}")
        if [ "${check_file_config}" == "$__IS_FALSE" ]; then
            __warn "Unable to read service key from ${APPOPTICS_CONFIG_FILE}. File not found."
            echo ""
        fi
        local config_service_key=$(sed -n -e '/\(key="ServiceKey"\)/p' "${APPOPTICS_CONFIG_FILE}" | grep -oP '(value=").*(")' | grep -oP '(").*(")' | sed 's/["\n]//g')
        echo ${config_service_key}
   fi
}
APPOPTICS_SERVICE_KEY_CONFIG_FILE=$(__read_service_key)

__find_dotnet_version()
{
    local dotnet_runtime_dir='/usr/share/dotnet/shared/Microsoft.NETCore.App'
    local check_dotnet_runtime_dir=$(__dir_exists "${dotnet_runtime_dir}")
    local dotnet_runtime_version=""
    if [ "${check_dotnet_runtime_dir}" == "${__IS_TRUE}" ]; then
        for dir in ${dotnet_runtime_dir}/*
        do
          dir=${dir%*/}
          dotnet_runtime_version="${dir##*/}"
        done
    fi
    echo ${dotnet_runtime_version}
}
DOTNET_RUNTIME_VERSION=$(__find_dotnet_version)
DOTNET_RUNTIME_MAJOR_VERSION=${DOTNET_RUNTIME_VERSION%.*.*}

__check_install()
{
    if [ "${APPOPTICS_IS_INSTALLED}" == "$__IS_FALSE" ]; then
        __error "${AGENT_DISPLAY_NAME} is not installed."
        return 
    fi

    local found_service_key=${__IS_FALSE}

    # Read service key from environment

    if [ -n "${APPOPTICS_SERVICE_KEY_ENV}" ]; then
        local masked_service_key_env=$(__mask_service_key ${APPOPTICS_SERVICE_KEY_ENV})
        __info "Service key set in APPOPTICS_SERVICE_KEY environment variable: ${masked_service_key_env}"
        found_service_key=${__IS_TRUE}
    else 
        __info "No service key set in APPOPTICS_SERVICE_KEY environment variable."
    fi

    # Read service key from agent.config 

    if [ -n "${APPOPTICS_SERVICE_KEY_CONFIG_FILE}" ]; then
        local masked_service_key=$(__mask_service_key ${APPOPTICS_SERVICE_KEY_CONFIG_FILE})
        __info "Found ${masked_service_key} service key inside agent.config..."
        found_service_key=${__IS_TRUE}
    else
        __info "No service key found inside agent.config."
    fi

    if [ "${found_service_key}" == "${__IS_FALSE}" ]; then
        __error "No service key set is in APPOPTICS_SERVICE_KEY environment variable or found in agent.config configuration file."
    fi

    # Check .NET Core profiler environment variables

    local profiler_is_enabled=""
    if [ -n "${CORECLR_ENABLE_PROFILING}" ]; then
        if [ "${CORECLR_ENABLE_PROFILING}" == "1" ]; then
            profiler_is_enabled="enabled"
        else
            profiler_is_enabled="disabled"
        fi
    else
        __warn "Environment variable CORECLR_ENABLE_PROFILING is not set. To enable ${AGENT_DISPLAY_NAME} profiler set CORECLR_ENABLE_PROFILING to 1".
    fi

    if [ -n "${CORECLR_PROFILER_PATH}" ]; then
        if [ "${CORECLR_PROFILER_PATH}" != "${APPOPTICS_PROFILER}" ]; then
            __error "Environment variable CORECLR_PROFILER_PATH is set a wrong value ${CORECLR_PROFILER_PATH}. To enable ${AGENT_DISPLAY_NAME} set CORECLR_PROFILER_PATH to ${APPOPTICS_PROFILER}."
        fi
    else
        __warn "Environment variable CORECLR_PROFILER_PATH is not set. To enable ${AGENT_DISPLAY_NAME} profiler set CORECLR_PROFILER_PATH to ${APPOPTICS_PROFILER}."
    fi

    if [ -n "${CORECLR_PROFILER}" ]; then
        if [ "${CORECLR_PROFILER}" == "${APPOPTICS_PROFILER_CLASS_ID}" ]; then
            __info "${AGENT_DISPLAY_NAME} profiler is ${profiler_is_enabled}."
        else
            __error "Found a different .NET Core profiler ${CORECLR_PROFILER}. Profiler ${CORECLR_PROFILER_PATH} is ${profiler_is_enabled}. "
        fi
    else
        __warn "Environment variable CORECLR_PROFILER is not set. To enable ${AGENT_DISPLAY_NAME} profiler you need to set CORECLR_PROFILER to ${APPOPTICS_PROFILER_CLASS_ID}."
    fi
}

__display_info_env()
{
    local name=${1}
    local value=${2}
    __info "${name} = ${value}"
}

__display_environment()
{
    __info "-------------------------------------------------------------"
    __info "Global Environment Variables:"

    __display_info_env "APPOPTICS_HOME_APM_DOTNET" "${APPOPTICS_HOME_APM_DOTNET}"
    __display_info_env "APPOPTICS_CONFIG" "${APPOPTICS_CONFIG}"
    __display_info_env "APPOPTICS_SERVICE_KEY" "${APPOPTICS_SERVICE_KEY}"

    __display_info_env "CORECLR_ENABLE_PROFILING" "${CORECLR_ENABLE_PROFILING}"
    __display_info_env "CORECLR_PROFILER" "${CORECLR_PROFILER}"
    __display_info_env "CORECLR_PROFILER_PATH" "${CORECLR_PROFILER_PATH}"
    __info "-------------------------------------------------------------"
}

__display_dotnet_info()
{
    __info "-------------------------------------------------------------"
    __info ".NET Core Runtime Information:"
    __info "Version: ${DOTNET_RUNTIME_VERSION}"
    CORECLR_ENABLE_PROFILING=0 dotnet --info
    __info "-------------------------------------------------------------"
}

__do_collect()
{
    local check_log_dir=$(__dir_exists ${APPOPTICS_LOG_DIR})
    if [ "${check_log_dir}" == "${__IS_FALSE}" ]; then
        __warn "${AGENT_DISPLAY_NAME} logging directory doesn't exist. No agent log files to be collected."
        return
    fi

    echo Creating package ${ARCHIVE_NAME} ...

    IFS=$'\n' x=($(find ${APPOPTICS_LOG_DIR} -type f -size +${BIG_FILE_SIZE}))
    len=${#x[@]}
    if [ "$len" -gt 0 ]; then
        __warn "Found files with size bigger than ${BIG_FILE_SIZE} in ${APPOPTICS_LOG_DIR}."
        for file in "${x[@]}"; do
            __warn "Size for $file is greater than ${BIG_FILE_SIZE}.";
        done
    fi

    # Copy log file to logging directory to be archived

    cp ${LOG_FILE} ${APPOPTICS_LOG_DIR}

    local package_file=${OUTPUT_DIR}/${ARCHIVE_NAME}

    pushd ${APPOPTICS_LOG_DIR}
    tar -zcvf ${package_file} .
    popd

    # Remove log file from logging directory

    logfilename=$(basename ${LOG_FILE})
    rm -f ${APPOPTICS_LOG_DIR}/${logfilename}

    # Check that archive file was created

    if [ ! -f "${package_file}" ]; then
        __error "Failed to create archive ${package_file}"
        exit ${FAILURE}
    fi

    echo ${package_file} created.
}

__display_information()
{
    local package_file=${OUTPUT_DIR}/${ARCHIVE_NAME}
    local archive_file_size_bytes=$(__file_size "${package_file}")
    local archive_file_size_formatted=$(__format_filesize ${archive_file_size_bytes})

    local log_file_size_bytes=$(__file_size "${LOG_FILE}")
    local log_file_size_formatted=$(__format_filesize ${log_file_size_bytes})
    __info "-------------------------------------------------------------"
    __info "Results:"
    __info "  Archive : $package_file Size: ${archive_file_size_formatted}"
    __info "  Log file: ${LOG_FILE} Size: ${log_file_size_formatted} "
    __info ""
    __info "Please email archive and log file to us:"
    __info "   email: technicalsupport@solarwinds.com"
    __info "-------------------------------------------------------------"
}

__check_parameters()
{
    local check_dir=$(__dir_exists ${OUTPUT_DIR})
    if [ "${check_dir}" == "${__IS_FALSE}" ]; then
        __error "Directory ${OUTPUT_DIR} specified by --output parameter doesn't exist."
        exit ${FAILURE}
    fi
}

main()
{
    # On Azure platform, log directory is to a different location.

    if [ "$IS_AZURE" == "${__IS_TRUE}" ]; then
        APPOPTICS_LOG_DIR=${HOME}/LogFiles/AppOptics
    fi

    # Check if log directory exists to collect logs

    local check_log_dir=$(__dir_exists ${APPOPTICS_LOG_DIR})
    if [ "${check_log_dir}" == "${__IS_FALSE}" ]; then
        __warn "${AGENT_DISPLAY_NAME} logging directory doesn't exist. No agent log files to be collected."
    fi

    __info "-------------------------------------------------------------"
    __info "Running script to collect $AGENT_NAME information..."
    __info ""
    __info "Computer name: `hostname` "
    __info "Current date : `echo $(date +'%Y-%m-%d %H:%M')` "
    __info ""
    __info "For detailed instructions, FAQ, and full documentation:"
    __info "   http://docs.appoptics.com"
    __info ""
    __info "If you have any questions or problems, please contact us:"
    __info "   email: technicalsupport@solarwinds.com"
    __info "-------------------------------------------------------------"

    __check_parameters
    __display_environment
    __display_dotnet_info
    __check_install
    __do_collect
    __display_information
}

main

echo Log file: ${LOG_FILE}
echo ${PROGNAME} done.

exit ${SUCCESS}
